@@ -1,14 +1,16 @@
 """Code run on the client side for unstaging complete Pulsar jobs."""
 import fnmatch
 from contextlib import contextmanager
-from logging import getLogger
-from os.path import join, relpath
 from json import loads
+from logging import getLogger
+from os.path import (
+    join,
+    relpath,
+)
 
 from ..action_mapper import FileActionMapper
 from ..staging import COMMAND_VERSION_FILENAME
 
-
 log = getLogger(__name__)
 
 
@@ -133,7 +135,7 @@ class ResultsCollector:
         )
 
     def __realized_dynamic_file_source_references(self):
-        references = []
+        references = {"filename": [], "extra_files": []}
 
         def record_references(from_dict):
             if isinstance(from_dict, list):
@@ -141,8 +143,8 @@ class ResultsCollector:
                     record_references(v)
             elif isinstance(from_dict, dict):
                 for k, v in from_dict.items():
-                    if k == "filename":
-                        references.append(v)
+                    if k in references:
+                        references[k].append(v)
                     if isinstance(v, (list, dict)):
                         record_references(v)
 
@@ -180,7 +182,7 @@ class ResultsCollector:
                 continue
             if self.client_outputs.dynamic_match(name):
                 collect = True
-            elif name in dynamic_file_source_references:
+            elif name in dynamic_file_source_references["filename"] or any(name.startswith(r) for r in dynamic_file_source_references["extra_files"]):
                 collect = True
 
             if collect:
