@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Script used to run an example job against a running Pulsar server.
 
 Exercises various features both the Pulsar client and server.
@@ -14,26 +13,22 @@ import tempfile
 import threading
 import time
 import traceback
-
 from collections import namedtuple
-from io import open
 
 from galaxy.tool_util.deps.dependencies import DependenciesDescription
 from galaxy.tool_util.deps.requirements import ToolRequirement
-from six import binary_type
 
 from pulsar.client import (
     build_client_manager,
-    ClientJobDescription,
-    ClientInputs,
-    ClientInput,
-    ClientOutputs,
     CLIENT_INPUT_PATH_TYPES,
+    ClientInput,
+    ClientInputs,
+    ClientJobDescription,
+    ClientOutputs,
     finish_job,
     PulsarOutputs,
     submit_job,
 )
-
 from .test_common import write_config
 
 TEST_SCRIPT = b"""# -*- coding: utf-8 -*-
@@ -79,6 +74,8 @@ metadata_dir = dirname(sys.argv[13])
 output5 = open(sys.argv[15], 'w')
 legacy_galaxy_json = int(sys.argv[16])
 output_metadata_path = join(metadata_dir, "metadata_output")
+tmp_dir_exists_path = join(".", "tmp_dir_test")
+home_dir_exists_path = join(".", "home_dir_test")
 try:
     assert_path_contents(sys.argv[2], "Hello world input!!@!")
     assert_path_contents(sys.argv[8], "INPUT_EXTRA_CONTENTS")
@@ -98,6 +95,13 @@ try:
     with open("gjson_refer", "w") as f: f.write('''gjson_refer_contents''')
     with open("gjson_refer2", "w") as f: f.write('''gjson_refer_contents2''')
     with open(output_metadata_path, "w") as f: f.write("meta output")
+    tmp_dir = getenv("_GALAXY_JOB_TMP_DIR", "fakepathnotreal")
+    tmp_dir_exists = str(exists(tmp_dir))
+    with open(tmp_dir_exists_path, "w") as f: f.write(f"tmp-{tmp_dir_exists}")
+    home_dir = getenv("_GALAXY_JOB_HOME_DIR", "fakepathnotreal")
+    home_dir_exists = str(exists(home_dir))
+    with open(home_dir_exists_path, "w") as f: f.write(f"home-{home_dir_exists}")
+
     output3.write(getenv("MOO", "moo_default"))
     output1_extras_path = "%s_files" % sys.argv[3][0:-len(".dat")]
     makedirs(output1_extras_path)
@@ -112,6 +116,7 @@ finally:
     output3.close()
     version_output.close()
     output4_index_path.close()
+sys.exit(4)
 """
 
 HELP_URL = "URL of the Pulsar web server to target."
@@ -129,14 +134,14 @@ HELP_JOB_ID = "Submit the Pulsar job with this 'external' id."
 HELP_DEBUG = "Enable debug log output from Pulsar client"
 
 EXPECTED_OUTPUT = b"hello world output"
-EXAMPLE_UNICODE_TEXT = u'єχαмρℓє συтρυт'
+EXAMPLE_UNICODE_TEXT = 'єχαмρℓє συтρυт'
 TEST_REQUIREMENT = ToolRequirement(name="dep1", version="1.1", type="package")
 TEST_DEPENDENCIES = DependenciesDescription(requirements=[TEST_REQUIREMENT])
 
 ClientInfo = namedtuple("ClientInfo", ["client", "client_manager"])
 
 
-class MockTool(object):
+class MockTool:
 
     def __init__(self, tool_dir):
         self.id = "client_test"
@@ -155,6 +160,7 @@ class TestRequiredFilesObject:
 def run(options):
     logging.basicConfig(level=logging.DEBUG)
     waiter = None
+    client_manager = None
     try:
         temp_directory = tempfile.mkdtemp(prefix='pulsar-check-client.')
         temp_index_dir = os.path.join(temp_directory, "idx", "bwa")
@@ -198,6 +204,11 @@ def run(options):
         temp_output_workdir_destination2 = os.path.join(temp_directory, "dataset_78.dat")
         temp_output_workdir2 = os.path.join(temp_work_dir, "rewrite_action_test")
 
+        temp_output_workdir_destination3 = os.path.join(temp_directory, "dataset_79.dat")
+        temp_output_workdir3 = os.path.join(temp_work_dir, "tmp_dir_test")
+        temp_output_workdir_destination4 = os.path.join(temp_directory, "dataset_80.dat")
+        temp_output_workdir4 = os.path.join(temp_work_dir, "home_dir_test")
+
         __write_to_file(temp_input_path, b"Hello world input!!@!")
         __write_to_file(temp_input_extra_path, b"INPUT_EXTRA_CONTENTS")
         __write_to_file(temp_input_metadata_path, b"INPUT METADATA CONTENTS...")
@@ -210,12 +221,12 @@ def run(options):
         __write_to_file("%s.fai" % temp_index_path, b"AGTC")
         __write_to_file(os.path.join(temp_index_dir_sibbling, "human_full_seqs"), b"AGTC")
 
-        empty_input = u"/foo/bar/x"
+        empty_input = "/foo/bar/x"
 
         test_unicode = getattr(options, "test_unicode", False)  # TODO Switch this in integration tests
         legacy_galaxy_json = getattr(options, "legacy_galaxy_json", False)
         cmd_text = EXAMPLE_UNICODE_TEXT if test_unicode else "Hello World"
-        command_line_params = (
+        command_line_arguments = (
             temp_tool_path,
             temp_config_path,
             temp_input_path,
@@ -235,7 +246,8 @@ def run(options):
             "1" if legacy_galaxy_json else "0",
         )
         assert os.path.exists(temp_index_path)
-        command_line = u'python %s "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s" "%s"' % command_line_params
+        quoted_args = (f'"{a}"' for a in command_line_arguments)
+        command_line = f"python {' '.join(quoted_args)} 2> ../metadata/tool_stderr > ../metadata/tool_stdout"
         config_files = [temp_config_path]
         client_inputs = []
         client_inputs.append(ClientInput(temp_input_path, CLIENT_INPUT_PATH_TYPES.INPUT_PATH))
@@ -254,6 +266,8 @@ def run(options):
             temp_output5_path,
             temp_output_workdir_destination,
             temp_output_workdir_destination2,
+            temp_output_workdir_destination3,
+            temp_output_workdir_destination4,
         ]
         client, client_manager = __client(temp_directory, options)
         waiter = Waiter(client, client_manager)
@@ -268,6 +282,8 @@ def run(options):
             work_dir_outputs=[
                 (temp_output_workdir, temp_output_workdir_destination),
                 (temp_output_workdir2, temp_output_workdir_destination2),
+                (temp_output_workdir3, temp_output_workdir_destination3),
+                (temp_output_workdir4, temp_output_workdir_destination4),
             ],
             output_files=output_files,
             version_file=temp_version_output_path,
@@ -283,15 +299,27 @@ def run(options):
             metadata_directory=temp_metadata_dir,
             **__extra_job_description_kwargs(options)
         )
-        submit_job(client, job_description)
+        job_id = submit_job(client, job_description)
+        if job_id:
+            client.job_id = job_id
         result_status = waiter.wait()
 
-        stdout = result_status["stdout"].strip()
-        assert "stdout output".startswith(stdout)
-        if hasattr(options, "maximum_stream_size"):
-            assert len(stdout) == options.maximum_stream_size
+        expecting_full_metadata = getattr(options, "expecting_full_metadata", True)
+        if expecting_full_metadata:
+            stdout = result_status["stdout"].strip()
+            std_streams_debug = f"actual stdout [{stdout}], actual stderr [{result_status['stderr']}]"
+            assert "stdout output".startswith(stdout), f"Standard output is not an initial substring of [stdout output], {std_streams_debug}"
 
-        __finish(options, client, client_outputs, result_status)
+            if hasattr(options, "maximum_stream_size"):
+                assert len(stdout) == options.maximum_stream_size
+
+        assert result_status["complete"] == "true"
+        if expecting_full_metadata:
+            assert result_status["returncode"] == 4, f"Expected exit code of 4, got {result_status['returncode']} - {std_streams_debug}"
+        if expecting_full_metadata:
+            __finish(options, client, client_outputs, result_status)
+        else:
+            client.clean()
         __assert_contents(temp_output_path, EXPECTED_OUTPUT, result_status)
         __assert_contents(temp_output2_path, cmd_text, result_status)
         if not legacy_galaxy_json:
@@ -303,6 +331,8 @@ def run(options):
         __assert_contents(os.path.join(temp_metadata_dir, "metadata_output"), b"meta output", result_status)
         if getattr(options, "test_rewrite_action", False):
             __assert_contents(temp_output_workdir_destination2, os.path.join(temp_directory, "shared2", "test1"), result_status)
+        __assert_contents(temp_output_workdir_destination3, "tmp-True", result_status)
+        __assert_contents(temp_output_workdir_destination4, "home-True", result_status)
         if job_description.env:
             __assert_contents(temp_output_workdir_destination, b"TEST_ENV_VALUE", result_status)
         __assert_contents(temp_version_output_path, b"1.0.1", result_status)
@@ -310,23 +340,30 @@ def run(options):
             __assert_contents(temp_output3_path, "moo_override", result_status)
         else:
             __assert_contents(temp_output3_path, "moo_default", result_status)
-        __assert_has_rewritten_bwa_path(client, temp_output4_path)
+        client_job_id = getattr(options, "job_id", "123456")
+        # Revise this test so it works for containers... or fix whatever is broken.
+        __assert_has_rewritten_bwa_path(client, client_job_id, temp_output4_path)
         if getattr(options, "explicit_tool_declarations", False):
             __assert_contents(temp_output5_path, str(True), result_status)
         __exercise_errors(options, client, temp_output_path, temp_directory)
-        client_manager.shutdown()
     except BaseException:
         if not options.suppress_output:
             traceback.print_exc()
         raise
     finally:
+        if client_manager:
+            client_manager.shutdown()
         if waiter is not None:
             waiter.shutdown()
         if getattr(options, 'cleanup', True):
-            shutil.rmtree(temp_directory)
+            try:
+                shutil.rmtree(temp_directory)
+            except PermissionError as e:
+                print(e)
+                print(f"Problem removing directory tree {temp_directory} due to a permission error.")
 
 
-class Waiter(object):
+class Waiter:
 
     def __init__(self, client, client_manager):
         self.client = client
@@ -345,7 +382,7 @@ class Waiter(object):
 
             self.client_manager.ensure_has_status_update_callback(on_update)
 
-    def wait(self, seconds=15):
+    def wait(self, seconds=120):
         final_status = None
         if not self.background:
             i = 0
@@ -373,27 +410,28 @@ class Waiter(object):
 
 def __assert_contents(path, expected_contents, pulsar_state):
     if not os.path.exists(path):
-        raise AssertionError("File %s not created. Final Pulsar response state [%s]" % (path, pulsar_state))
-    if isinstance(expected_contents, binary_type):
+        raise AssertionError("File {} not created. Final Pulsar response state [{}]".format(path, pulsar_state))
+    if isinstance(expected_contents, bytes):
         file = open(path, 'rb')
     else:
-        file = open(path, 'r', encoding="utf-8")
+        file = open(path, encoding="utf-8")
     try:
         contents = file.read()
         if contents != expected_contents:
-            message = "File (%s) contained invalid contents [%s]." % (path, contents)
-            message = "%s Expected contents [%s]. Final Pulsar response state [%s]" % (message, expected_contents, pulsar_state)
+            message = "File ({}) contained invalid contents [{}].".format(path, contents)
+            message = "{} Expected contents [{}]. Final Pulsar response state [{}]".format(message, expected_contents, pulsar_state)
             raise AssertionError(message)
     finally:
         file.close()
 
 
-def __assert_has_rewritten_bwa_path(client, temp_output4_path):
-    if client.default_file_action != "none":
-        rewritten_index_path = open(temp_output4_path, 'r', encoding='utf-8').read()
+def __assert_has_rewritten_bwa_path(client, job_id, temp_output4_path):
+    if client.default_file_action != "none" and job_id:
+        rewritten_index_path = open(temp_output4_path, encoding='utf-8').read()
         # Path written to this file will differ between Windows and Linux.
-        if re.search(r"%s[/\\]unstructured[/\\]\w+[/\\]bwa[/\\]human.fa" % client.job_id, rewritten_index_path) is None:
-            raise AssertionError("[%s] does not contain rewritten path." % rewritten_index_path)
+        patt = r"%s[/\\]unstructured[/\\]\w+[/\\]bwa[/\\]human.fa" % job_id
+        if re.search(patt, rewritten_index_path) is None:
+            raise AssertionError("[%s] does not contain rewritten path [%s]." % (patt, rewritten_index_path))
 
 
 def __exercise_errors(options, client, temp_output_path, temp_directory):
@@ -430,7 +468,7 @@ def __client(temp_directory, options):
     if default_file_action in ["remote_scp_transfer", "remote_rsync_transfer"]:
         test_key = os.environ["PULSAR_TEST_KEY"]
         if not test_key.startswith("----"):
-            test_key = open(test_key, "r").read()
+            test_key = open(test_key).read()
         client_options["ssh_key"] = test_key
         client_options["ssh_user"] = os.environ.get("USER")
         client_options["ssh_port"] = 22
@@ -456,13 +494,15 @@ def extract_client_options(options):
     if default_file_action:
         client_options["default_file_action"] = default_file_action
     if hasattr(options, "jobs_directory"):
-        client_options["jobs_directory"] = getattr(options, "jobs_directory")
+        client_options["jobs_directory"] = options.jobs_directory
     if hasattr(options, "files_endpoint"):
-        client_options["files_endpoint"] = getattr(options, "files_endpoint")
+        client_options["files_endpoint"] = options.files_endpoint
     if hasattr(options, "k8s_enabled"):
-        client_options["k8s_enabled"] = getattr(options, "k8s_enabled")
+        client_options["k8s_enabled"] = options.k8s_enabled
+    if hasattr(options, "tes_url"):
+        client_options["tes_url"] = options.tes_url
     if hasattr(options, "container"):
-        client_options["container"] = getattr(options, "container")
+        client_options["container"] = options.container
     return client_options
 
 
@@ -488,7 +528,12 @@ def client_manager_from_args(options):
         manager_args['transport'] = options.transport
     if getattr(options, 'manager_url', None):
         manager_args['amqp_url'] = options.manager_url
-    return build_client_manager(**manager_args)
+    if getattr(options, 'tes_url', None):
+        manager_args['tes_url'] = options.tes_url
+    if getattr(options, "k8s_enabled", None):
+        manager_args['k8s_enabled'] = options.k8s_enabled
+    cm = build_client_manager(**manager_args)
+    return cm
 
 
 def __write_to_file(path, contents):
@@ -496,8 +541,8 @@ def __write_to_file(path, contents):
     if not os.path.exists(dirname):
         os.makedirs(dirname)
     with open(path, "wb") as f:
-        if not isinstance(contents, binary_type):
-            contents = binary_type(contents, "UTF-8")
+        if not isinstance(contents, bytes):
+            contents = bytes(contents, "UTF-8")
         f.write(contents)
 
 
@@ -540,7 +585,7 @@ def __finish(options, client, client_outputs, result_status):
     if failed:
         failed_message_template = "Failed to complete job correctly, final status %s, finish exceptions %s."
         failed_message = failed_message_template % (result_status, failed)
-        assert False, failed_message
+        raise AssertionError(failed_message)
 
 
 def main(argv=None):
