@@ -1,34 +1,38 @@
 """Utilities allowing for high-level testing throughout Pulsar."""
-from __future__ import print_function
 
-import traceback
+import os
+import configparser
 import sys
 import threading
-import os
-
+import traceback
 from contextlib import contextmanager
 from stat import S_IXGRP, S_IXOTH
 from os import pardir, stat, chmod, access, X_OK, pathsep, environ
-from os import makedirs, listdir
+from os import makedirs, listdir, system
 from os.path import join, dirname, isfile, split
-from os.path import exists
+from pathlib import Path
 from tempfile import mkdtemp
 from shutil import rmtree
+from typing import (
+    Any,
+    Dict,
+    Optional,
+)
+
 import time
 
-import webob
+import pytest
 from webtest import TestApp
 from webtest.http import StopableWSGIServer
 
-import galaxy.util
-import pulsar.util
+from galaxy.job_metrics import NULL_JOB_INSTRUMENTER
 from galaxy.util.bunch import Bunch
-from galaxy.jobs.metrics import NULL_JOB_INSTRUMENTER
+from simplejobfiles.app import JobFilesApp
 
 from pulsar.managers.util import drmaa
 from pulsar.tools import ToolBox
 from pulsar.managers.base import JobDirectory
-from pulsar.web.framework import file_response
+from pulsar.user_auth.manager import UserAuthManager
 
 from unittest import TestCase, skip
 
@@ -42,6 +46,14 @@ import stopit
 from functools import wraps
 
 
+class MarkGenerator:
+    def __getattr__(self, name):
+        return getattr(pytest.mark, name)
+
+
+mark = MarkGenerator()
+
+
 def timed(timeout):
     def outer_wrapper(f):
         @wraps(f)
@@ -56,7 +68,7 @@ def timed(timeout):
     return outer_wrapper
 
 
-INTEGRATION_MAXIMUM_TEST_TIME = 15
+INTEGRATION_MAXIMUM_TEST_TIME = 120
 integration_test = timed(INTEGRATION_MAXIMUM_TEST_TIME)
 
 TEST_DIR = dirname(__file__)
@@ -83,7 +95,7 @@ def get_test_tool():
     return get_test_toolbox().get_tool("tool1")
 
 
-class TestManager(object):
+class TestManager:
 
     def setup_temp_directory(self):
         self.temp_directory = temp_directory_persist(prefix='test_manager_')
@@ -123,7 +135,7 @@ def test_manager():
     manager.cleanup_temp_directory()
 
 
-class TestAuthorization(object):
+class TestAuthorization:
 
     def __init__(self):
         self.allow_setup = True
@@ -148,7 +160,7 @@ class TestAuthorization(object):
             raise Exception
 
 
-class TestDependencyManager(object):
+class TestDependencyManager:
 
     def dependency_shell_commands(self, requirements, **kwds):
         return []
@@ -160,22 +172,30 @@ class BaseManagerTestCase(TestCase):
         self.app = minimal_app_for_managers()
         self.staging_directory = self.app.staging_directory
         self.authorizer = self.app.authorizer
+        self.user_auth_manager = self.app.user_auth_manager
 
     def tearDown(self):
         rmtree(self.staging_directory)
 
     @nottest
-    def _test_simple_execution(self, manager):
-        command = """python -c "import sys; sys.stdout.write(\'Hello World!\'); sys.stderr.write(\'moo\')" """
+    def _test_simple_execution(self, manager, timeout=None):
+        command = """
+python -c "import sys; sys.stdout.write(\'Hello World!\'); sys.stdout.flush(); sys.stderr.write(\'moo\'); sys.stderr.flush()" \
+2> ../metadata/tool_stderr > ../metadata/tool_stdout"""
         job_id = manager.setup_job("123", "tool1", "1.0.0")
         manager.launch(job_id, command)
+
+        time_end = None if timeout is None else time.time() + timeout
         while manager.get_status(job_id) not in ['complete', 'cancelled']:
-            pass
-        self.assertEquals(manager.stderr_contents(job_id), b'moo')
-        self.assertEquals(manager.stdout_contents(job_id), b'Hello World!')
-        self.assertEquals(manager.return_code(job_id), 0)
+            if time_end and time.time() > time_end:
+                raise Exception("Timeout.")
+        self.assertEqual(manager.job_stderr_contents(job_id), b"")
+        self.assertEqual(manager.job_stdout_contents(job_id), b"")
+        self.assertEqual(manager.stderr_contents(job_id), b"moo")
+        self.assertEqual(manager.stdout_contents(job_id), b"Hello World!")
+        self.assertEqual(manager.return_code(job_id), 0)
         manager.clean(job_id)
-        self.assertEquals(len(listdir(self.staging_directory)), 0)
+        self.assertEqual(len(listdir(self.staging_directory)), 0)
 
     def _test_cancelling(self, manager):
         job_id = manager.setup_job("124", "tool1", "1.0.0")
@@ -211,13 +231,27 @@ def minimal_app_for_managers():
     staging_directory = temp_directory_persist(prefix='minimal_app_')
     rmtree(staging_directory)
     authorizer = TestAuthorizer()
+    user_auth_manager = get_test_user_auth_manager()
     return Bunch(staging_directory=staging_directory,
                  authorizer=authorizer,
                  job_metrics=NullJobMetrics(),
-                 dependency_manager=TestDependencyManager())
+                 dependency_manager=TestDependencyManager(),
+                 user_auth_manager=user_auth_manager,
+                 object_store=object())
 
 
-class NullJobMetrics(object):
+def get_test_user_auth_manager():
+    config = {"user_auth": {"authentication": [{"type": "allow_all"}], "authorization": [{"type": "allow_all"}]}}
+    return UserAuthManager(config)
+
+
+def get_failing_user_auth_manager():
+    config = {"user_auth": {"authentication": [{"type": "allow_all"}],
+                            "authorization": [{"type": "userlist", "userlist_allowed_users": []}]}}
+    return UserAuthManager(config)
+
+
+class NullJobMetrics:
 
     def __init__(self):
         self.default_job_instrumenter = NULL_JOB_INSTRUMENTER
@@ -225,15 +259,13 @@ class NullJobMetrics(object):
 
 @nottest
 @contextmanager
-def server_for_test_app(app):
-    try:
-        from paste.exceptions.errormiddleware import ErrorMiddleware
-        error_app = ErrorMiddleware(app.app, debug=True, error_log="errors.log")
-        server = StopableWSGIServer.create(error_app)
-    except ImportError:
-        # paste.exceptions not available for Python 3.
-        error_app = app.app
-        server = StopableWSGIServer.create(error_app)
+def server_for_test_app(test_app):
+    app = test_app.app
+    create_kwds = {
+    }
+    if os.environ.get("PULSAR_TEST_FILE_SERVER_HOST"):
+        create_kwds["host"] = os.environ.get("PULSAR_TEST_FILE_SERVER_HOST")
+    server = StopableWSGIServer.create(app, **create_kwds)
     try:
         server.wait()
         yield server
@@ -254,7 +286,10 @@ def test_pulsar_server(global_conf={}, app_conf={}, test_conf={}):
             yield test_pulsar_server
 
 
-class RestartablePulsarAppProvider(object):
+test_pulsar_server.__test__ = False
+
+
+class RestartablePulsarAppProvider:
 
     def __init__(self, global_conf={}, app_conf={}, test_conf={}, web=True):
         self.staging_directory = temp_directory_persist(prefix='staging_')
@@ -266,11 +301,11 @@ class RestartablePulsarAppProvider(object):
     @contextmanager
     def new_app(self):
         with test_pulsar_app(
-            self.global_conf,
-            self.app_conf,
-            self.test_conf,
-            staging_directory=self.staging_directory,
-            web=self.web,
+                self.global_conf,
+                self.app_conf,
+                self.test_conf,
+                staging_directory=self.staging_directory,
+                web=self.web,
         ) as app:
             yield app
 
@@ -293,11 +328,11 @@ def restartable_pulsar_app_provider(**kwds):
 @nottest
 @contextmanager
 def test_pulsar_app(
-    global_conf={},
-    app_conf={},
-    test_conf={},
-    staging_directory=None,
-    web=True,
+        global_conf={},
+        app_conf={},
+        test_conf={},
+        staging_directory=None,
+        web=True,
 ):
     clean_staging_directory = False
     if staging_directory is None:
@@ -328,6 +363,9 @@ def test_pulsar_app(
                 pass
 
 
+test_pulsar_app.__test__ = False
+
+
 @contextmanager
 def _yield_app(global_conf, app_conf, test_conf, web):
     # Yield either wsgi webapp of the underlying pulsar
@@ -369,7 +407,8 @@ def skip_unless_module(module):
     available = True
     try:
         __import__(module)
-    except ImportError:
+    except (ImportError, RuntimeError):
+        # drmaa raises RuntimeError if DRMAA_LIBRARY_PATH is unset
         available = False
     if available:
         return lambda func: func
@@ -400,7 +439,6 @@ def skip_without_drmaa(f):
 
 
 def _which(program):
-
     def is_exe(fpath):
         return isfile(fpath) and access(fpath, X_OK)
 
@@ -418,7 +456,7 @@ def _which(program):
     return None
 
 
-class TestAuthorizer(object):
+class TestAuthorizer:
 
     def __init__(self):
         self.authorization = TestAuthorization()
@@ -427,51 +465,29 @@ class TestAuthorizer(object):
         return self.authorization
 
 
-class JobFilesApp(object):
-
-    def __init__(self, root_directory=None):
-        self.root_directory = root_directory
-
-    def __call__(self, environ, start_response):
-        req = webob.Request(environ)
-        params = req.params.mixed()
-        method = req.method
-        if method == "POST":
-            resp = self._post(req, params)
-        elif method == "GET":
-            resp = self._get(req, params)
-        else:
-            raise Exception("Unhandled request method %s" % method)
-        return resp(environ, start_response)
-
-    def _post(self, request, params):
-        path = params['path']
-        if not galaxy.util.in_directory(path, self.root_directory):
-            assert False, "%s not in %s" % (path, self.root_directory)
-        parent_directory = dirname(path)
-        if not exists(parent_directory):
-            makedirs(parent_directory)
-        pulsar.util.copy_to_path(params["file"].file, path)
-        return webob.Response(body='')
-
-    def _get(self, request, params):
-        path = params['path']
-        if not galaxy.util.in_directory(path, self.root_directory):
-            assert False, "%s not in %s" % (path, self.root_directory)
-        return file_response(path)
-
-
 @contextmanager
 def files_server(directory=None):
-    if not directory:
-        with temp_directory() as directory:
+    external_url = os.environ.get("PULSAR_TEST_EXTERNAL_JOB_FILES_URL")
+    if external_url:
+        if directory is None:
+            directory = os.environ.get("PULSAR_TEST_EXTERNAL_JOB_FILES_DIRECTORY")
+            if directory:
+                yield Bunch(application_url=external_url), directory
+            else:
+                with temp_directory() as directory:
+                    yield Bunch(application_url=external_url), directory
+        else:
+            yield Bunch(application_url=external_url)
+    else:
+        if not directory:
+            with temp_directory() as directory:
+                app = TestApp(JobFilesApp(directory))
+                with server_for_test_app(app) as server:
+                    yield server, directory
+        else:
             app = TestApp(JobFilesApp(directory))
             with server_for_test_app(app) as server:
-                yield server, directory
-    else:
-        app = TestApp(JobFilesApp(directory))
-        with server_for_test_app(app) as server:
-            yield server
+                yield server
 
 
 def dump_other_threads():
@@ -488,8 +504,7 @@ def dump_other_threads():
 
 # Extracted from: https://github.com/python/cpython/blob/
 # 937ee9e745d7ff3c2010b927903c0e2a83623324/Lib/test/support/__init__.py
-class EnvironmentVarGuard(object):
-
+class EnvironmentVarGuard:
     """Class to help protect the environment variable properly.  Can be used as
     a context manager."""
 
@@ -539,3 +554,55 @@ class EnvironmentVarGuard(object):
             else:
                 self._environ[k] = v
         os.environ = self._environ
+
+
+class IntegrationTestConfiguration:
+    _test_suffix: Optional[str]
+    _app_conf_dict: Dict[str, Any]
+    _test_conf_dict: Dict[str, Any]
+
+    def __init__(self, tmp_path: Path, test_suffix: Optional[str] = None):
+        self._app_conf_dict = {}
+        self._test_conf_dict = {}
+        self._tmp_path = tmp_path
+        self.__setup_dependencies()
+        self._test_suffix = test_suffix
+
+    def __setup_dependencies(self):
+        dependencies_dir = self._tmp_path / "dependencies"
+        dep1_directory = dependencies_dir / "dep1" / "1.1"
+        makedirs(dep1_directory)
+        try:
+            # Let external users read/execute this directory for run as user
+            # test.
+            system("chmod 755 %s" % self._tmp_path)
+            system("chmod -R 755 %s" % dependencies_dir)
+        except Exception as e:
+            print(e)
+        env_file = dep1_directory / "env.sh"
+        env_file.write_text("MOO=moo_override; export MOO")
+        self._app_conf_dict["tool_dependency_dir"] = str(dependencies_dir)
+
+    def write_job_conf_props(self, job_conf_props: Optional[Dict[str, str]]):
+        if job_conf_props:
+            job_conf_props = job_conf_props.copy()
+            job_conf = self._tmp_path / "job_managers.ini"
+            config = configparser.ConfigParser()
+            section_name = "manager:_default_"
+            config.add_section(section_name)
+            for key, value in job_conf_props.items():
+                config.set(section_name, key, value)
+            with open(job_conf, "w") as configf:
+                config.write(configf)
+
+            self._app_conf_dict["job_managers_config"] = job_conf
+
+    def set_app_conf_props(self, **kwd):
+        self._app_conf_dict.update(**kwd)
+
+    def set_test_conf_props(self, **kwd):
+        self._test_conf_dict.update(**kwd)
+
+    @property
+    def test_suffix(self) -> str:
+        return self._test_suffix or ""
